pub enum Error {
Show 13 variants
Unauthorized,
NotFound {
resource: String,
},
InternalServerError {
source: Option<Report>,
},
Sqlx {
source: Error,
},
MigrationError {
source: MigrateError,
},
IdentityNotFound {
key_id: String,
},
PayloadTooLarge,
MissingHeader {
header: String,
},
InvalidHeader {
header: String,
},
Whatever {
message: String,
source: Option<Report>,
},
InvalidParameter {
message: String,
},
InvalidMethod {
message: String,
},
MissingParameter {
message: String,
},
}Expand description
The main error enum that represents all possible errors in the application. Each variant includes context-specific information and appropriate error messages.
Variants
NotFound
InternalServerError
Sqlx
MigrationError
Fields

source: MigrateErrorIdentityNotFound
PayloadTooLarge
MissingHeader
InvalidHeader
Whatever
InvalidParameter
InvalidMethod
MissingParameter
Implementations
Sourceimpl Error
Convenience methods for creating common error types
impl Error
Convenience methods for creating common error types
Sourcepub fn internal(e: impl Into<Report>) -> Self
pub fn internal(e: impl Into<Report>) -> Self
Creates a new internal server error with a source error
Sourcepub fn not_found(resource: impl Into<String>) -> Self
pub fn not_found(resource: impl Into<String>) -> Self
Creates a not found error for a generic resource
pub fn invalid_parameter(message: impl Into<String>) -> Self
pub fn missing_parameter(message: impl Into<String>) -> Self
Sourcepub fn queue_not_found(
queue: impl Into<String>,
namespace: impl Into<String>,
) -> Self
pub fn queue_not_found( queue: impl Into<String>, namespace: impl Into<String>, ) -> Self
Creates a not found error specifically for queues within a namespace
Sourcepub fn namespace_not_found(namespace: impl Into<String>) -> Self
pub fn namespace_not_found(namespace: impl Into<String>) -> Self
Creates a not found error specifically for namespaces